use diesel::prelude::*;

#[derive(Queryable, Selectable)]
#[diesel(table_name = crate::schema::users)]
#[diesel(check_for_backend(diesel::sqlite::Sqlite))]
pub struct User {
    pub id: i32,
    pub handle: String,
    pub password: String,
}

impl User {
    pub fn set_password(&mut self, password: &String) -> bool {
        let ok = bcrypt::hash(password, bcrypt::DEFAULT_COST);
        match ok {
            Ok(hash) => {
                self.password = hash;
                return true;
            }
            Err(e) => {
                if self.handle.is_empty() {
                    log::warn!("User.set_password >> {}", e);
                } else {
                    log::warn!("User '{}'.set_password >> {}", self.handle.clone(), e);
                }
                return false;
            }
        }
    }
    pub fn verify_password(&mut self, password: &String) -> bool {
        let ok = bcrypt::verify(password, &self.password);
        match ok {
            Ok(ok) => ok,
            Err(e) => {
                if self.handle.is_empty() {
                    log::warn!("User.verify_password >> {}", e);
                } else {
                    log::warn!("User '{}'.verify_password >> {}", self.handle.clone(), e);
                }
                return false;
            }
        }
    }
    pub fn password_cost(&self) -> u32 {
        let work = self.password.clone();
        let parts: Vec<&str> = work.split("$").collect();
        let cost = parts[2].to_string().parse::<u32>();
        match cost {
            Ok(c) => c,
            Err(e) => {
                if self.handle.is_empty() {
                    log::warn!("User.password_cost >> {} > '{}' of '{}'", e, parts[2], work);
                } else {
                    log::warn!("User '{}'.password_cost >> {} > '{}' of '{}'", self.handle.clone(), e, parts[2], work);
                }
                return 0;
            }
        }
    }
}

use crate::schema::users;

#[derive(Insertable)]
#[diesel(table_name = users)]
pub struct InsertUser {
    pub handle: String,
    pub password: String,
}

impl InsertUser {
    pub fn new(name: &String, password: &String) -> Self {
        let ok = bcrypt::hash(password, bcrypt::DEFAULT_COST);
        match ok {
            Ok(hash) => {
                Self{
                    handle: name.clone(),
                    password: hash,
                }
            }
            Err(e) => {
                log::error!("InsertUser '{}'.new >> {}", name.clone(), e);
                Self {
                    handle: name.clone(),
                    password: String::new()
                }
            }
        }
    }
}

pub fn create_user(conn: &mut SqliteConnection, name: &String, password: &String) -> User {
    use crate::schema::users;

    let new_user = InsertUser::new(name, password);

    diesel::insert_into(users::table)
        .values(&new_user)
        .returning(User::as_returning())
        .get_result(conn)
        .expect("Error saving new post")
}
